static char sccsid[] = "%Z%%M%  %I%  %W% %G% %U%";
/*
 *   (C) COPYRIGHT International Business Machines Corp. 1999
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/*
 *   Create User Dictionary
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/param.h>
#include <sys/stat.h>
#include "kingapi.h"
#include "transenv.h"

#define VERSION			"1.0.0.17"

void usage(void)
{
    const char* msg =
	"Usage: addudict [-u usrdictpath] [-n usrdictname] [-v] [-|csvfile]\n"
	"         -u   specify a path for user dictionary\n"
	"         -n   specify a file name of user dictionary (ex: usr)\n"
	"         -s   specify a path of system dictionary\n"
	"         -help      produce this help message\n"
	"         -version   report kingtr version\n"
	"         csvfile    CSV (Comma Separated Value format) file name\n";
    printf(msg);
    exit(0);
}

int
main(int argc, char* argv[])
{
    char* usrdictpath = NULL;
    char* usrdictname = NULL;
    char* csvfilename = NULL;
    char* sysdic = NULL;
    
    TransEnv env;
    if (!read_kingrc(USER_CONFIG_FILE, &env))
	(void)read_kingrc(SYSTEM_CONFIG_FILE, &env);

    while (--argc > 0) {
	char* p = *++argv;
	if (!strcmp(p, "-n")) {
	    usrdictname = *++argv;
	    argc--;
	}

	else if (!strcmp(p, "-u")) {
	    usrdictpath = *++argv;
	    argc--;
	}

	else if (!strcmp(p, "-s")) {
	    sysdic = *++argv;
	    argc--;
	    if (!sysdic || !*sysdic)
		usage();
	    if (env.SystemDictDirectory != NULL)
		delete[] env.SystemDictDirectory;
	    env.SystemDictDirectory = expand_file_name(sysdic);
	}

	else if (!strcmp(p, "-h") || !strcmp(p, "-help")
		 || !strcmp(p, "--help")) {
	    usage();
	}

	else if (!strcmp(p, "-version") || !strcmp(p, "--version")) {
	    printf("addudict version %s\n", VERSION);
	    exit(0);
	}

	else if (p[0] == '-' && p[1]) {
	    printf("Invalid option %s\n", p);
	    usage();
	}

	else {
	    if (p[0] != '-')
		csvfilename = p;
	}
    }


    char udict[MAXPATHLEN];
    if (usrdictpath)
	strcpy(udict, usrdictpath);
    else {
        usrdictpath = getenv("KINGUSRDICTPATH");
	if (usrdictpath)
	    strcpy(udict, usrdictpath);
	else {
	    strcpy(udict, getenv("HOME"));
	    strcat(udict, "/.king");
	}
    }

    struct stat sbuf;
    if (stat(udict, &sbuf) < 0) { // udict not found
	if (mkdir(udict, 0775) != 0) {
	    fprintf(stderr, "Can't create directory %s.\n", udict);
	    exit(1);
	}
    }

    if (usrdictname) {
	strcat(udict, "/");
	strcat(udict, usrdictname);
    }
    else
	strcat(udict, "/usr");

    int num_registered;
    char* errmsg = NULL;
    if ((num_registered = KingAddUserDict(env.SystemDictDirectory,
				   csvfilename, udict, &errmsg)) == -1) {
        fprintf(stderr, "%s\n", KingGetLastError());
	delete[] errmsg;
        exit(1);
    }
    else {
	int ret = 0;
	if (errmsg) {
	    printf("%s\n", errmsg);
	    delete[] errmsg;
	    ret = 1;
	}
        if (num_registered == 0)
	    printf("no entry added to %s.\n", udict);
        else if (num_registered == 1)
	    printf("%d entry added to %s.\n", num_registered, udict);
        else
	    printf("%d entries added to %s.\n", num_registered, udict);
	return ret;
    }
}
