static char sccsid[] = "%Z%%M%  %I%  %W% %G% %U%";
/*
 *   (C) COPYRIGHT International Business Machines Corp. 1999
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/*
 *   Read Translation Environment
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>
#include "transenv.h"

char*
expand_file_name(const char* path)
{
    char buf[1024];
    char* bufp = buf;
    const char* p = path;

    if (p[0] == '~') {
	if (p[1] == '/') {
	    char* home = getenv("HOME");
	    if (home) {
		strcpy(bufp, home);
		bufp += strlen(home);
		p++;
	    }
	    else {
		uid_t uid = getuid();
		struct passwd* pw = (struct passwd *)getpwuid(uid);
		if (pw) {
		    strcpy(bufp, pw->pw_dir);
		    bufp += strlen(pw->pw_dir);
		    p++;
		}
		else {
		    *bufp++ = *p++;
		}
	    }
	}
	else {
	    char user[64];
	    char* userp = user;
	    const char* q;
	    for (q = p + 1; *q && (*q != '/'); q++)
		*userp++ = *q;
	    *userp = '\0';
	    struct passwd* pw = (struct passwd *)getpwnam(user);
	    if (pw) {
		strcpy(bufp, pw->pw_dir);
		bufp += strlen(pw->pw_dir);
		p = q;
	    }
	    else {
		*bufp++ = *p++;
	    }
	}
    }

    while (*p)
	*bufp++ = *p++;
    *bufp = '\0';
    char* result = new char[strlen(buf) + 1];
    strcpy(result, buf);
    return result;
}

char*
expand_file_names(const char* path_list)
{
    char* p = new char[strlen(path_list) + 1];
    strcpy(p, path_list);
    char* top = NULL;
    char* result = NULL;
    for (int i = 0; p; i++) {
	top = p;
	p = strchr(p, ',');
	if (p)
	    *p++ = '\0';
	char* fullpath = expand_file_name(top);
	int len = strlen(fullpath);
	if (result)
	    len += strlen(result) + 1;	// +1 for ','
	char* tmp = new char[len + 1];
	tmp[0] = '\0';
	if (result) {
	    strcpy(tmp, result);
	    strcat(tmp, ",");
	}
	strcat(tmp, fullpath);
	delete[] fullpath;
	delete[] result;
	result = tmp;
    }
    delete[] p;
    return result;
}

int
read_kingrc(const char* file, TransEnv* env)
{
    FILE* fp;
    char buf[4096];
    char* expanded_file = expand_file_name(file);
    if ((fp = fopen(expanded_file, "r")) == NULL) {
	delete[] expanded_file;
	return 0;
    }
    delete[] expanded_file;
    while (fgets(buf, sizeof(buf), fp) != NULL) {
	if (!isalpha(buf[0]))
	    continue;
	char* val = NULL;
	for (char* p = buf; *p; p++) {
	    if (*p == ' ' || *p == '\t') {
		*p++ = '\0';
		while (*p == ' ' || *p == '\t')
		    ;
		if (*p != '\0' && *p != '\n' && *p != '\r') {
		    val = p;
		    p = strchr(val, '\n');
		    if (p)
			*p = '\0';
		    p = strchr(val, '\r');
		    if (p)
			*p = '\0';
		    break;
		}
	    }
	}
	if (!val)
	    continue;
	if (!env->SystemDictDirectory && !strcmp(buf, "SystemDictDirectory")) {
	    env->SystemDictDirectory = expand_file_name(val);
	}
	if (!env->Dictionaries && !strcmp(buf, "Dictionaries")) {
	    env->Dictionaries = expand_file_names(val);
	}
	if (!env->OutputStyle && !strcmp(buf, "OutputStyle")) {
	    env->OutputStyle = new char[strlen(val) + 1];
	    strcpy(env->OutputStyle, val);
	}
	if (!env->ParamThink && !strcmp(buf, "ParamThink")) {
	    env->ParamThink = new char[strlen(val) + 1];
	    strcpy(env->ParamThink, val);
	}
	if (!env->ParamStyle && !strcmp(buf, "ParamStyle")) {
	    env->ParamStyle = new char[strlen(val) + 1];
	    strcpy(env->ParamStyle, val);
	}
	if (!env->ParamPunct && !strcmp(buf, "ParamPunct")) {
	    env->ParamPunct = new char[strlen(val) + 1];
	    strcpy(env->ParamPunct, val);
	}
	if (!env->ParamNum && !strcmp(buf, "ParamNum")) {
	    env->ParamNum = new char[strlen(val) + 1];
	    strcpy(env->ParamNum, val);
	}
	if (!env->ParamUnitConv && !strcmp(buf, "ParamUnitConv")) {
	    env->ParamUnitConv = new char[strlen(val) + 1];
	    strcpy(env->ParamUnitConv, val);
	}
	if (!env->ParamKanji && !strcmp(buf, "ParamKanji")) {
	    env->ParamKanji = new char[strlen(val) + 1];
	    strcpy(env->ParamKanji, val);
	}
	if (!env->HttpProxyHost && !strcmp(buf, "HttpProxyHost")) {
	    env->HttpProxyHost = new char[strlen(val) + 1];
	    strcpy(env->HttpProxyHost, val);
	}
	if (!env->HttpProxyPort && !strcmp(buf, "HttpProxyPort")) {
	    env->HttpProxyPort = new char[strlen(val) + 1];
	    strcpy(env->HttpProxyPort, val);
	}
    }
    fclose(fp);
    return 1;
}
